{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{%- extends config.ACCOUNTS_COVER_TEMPLATE %}
{% from "invenio_accounts/_macros.html" import render_field, form_errors %}
{%- set messages = get_flashed_messages(with_categories=true) -%}

{% block page_body %}
<div class="ui padded segments big form">
  <div class="ui segment">
    {%- block form_header %}
      <h3 class="ui reset-password header">{{_('Reset Password')}}</h3>
    {%- endblock form_header %}

    {%- if messages %}
      {%- for category, message in messages %}
        <p>{{ message }}</p>
      {%- endfor %}
    {%- else %}
      {%- with form = reset_password_form %}
      <form action="{{ url_for_security('reset_password', token=reset_password_token) }}" method="POST" name="reset_password_form">
        {{ form.hidden_tag() }}
        {{ render_field(form.password, icon="lock icon", autofocus=True, errormsg='password' in form.errors) }}
        {{ render_field(form.password_confirm, icon="lock icon", autofocus=False,  errormsg='password_confirm' in form.errors) }}
        <button type="submit" class="ui fluid large submit primary button">{{_('Reset Password')}}</button>
      </form>
      {%- endwith %}
    {%- endif %}
  </div>

  {%- if current_user.is_anonymous %}
    <div class="ui secondary segment">
      <h4><a href="{{url_for('security.login')}}">{{_('Log in')}}</a>{% if security.registerable %}
        {{_('or')}} <a href="{{url_for('security.register')}}">{{_('Sign up')}}</a>{% endif %}</h4>
    </div>
  {%- endif %}
</div>
{% endblock page_body %}
