{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2018 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{% macro render_field(field, icon="", autofocus=False, errormsg=True, size="form-group-lg") %}
  <div class="form-group {{size}}{% if icon %} has-feedback{% endif %}{% if field.errors and errormsg %} has-error{% endif %}">
    {%- set extras = dict(autofocus="") if autofocus else dict() %}
    {{field(class_="form-control", placeholder=_(field.label.text | string), **extras)}}
    {%- if icon %}
    <i class="{{icon}} form-control-feedback" aria-hidden="true" ></i>
    {%- endif%}
  </div>
  {%- if field.errors and errormsg %}
  <div class="text-danger text-left" role="alert">
    {%- for error in field.errors %}
    <p>{{error}}</p>
    {%- endfor %}
  </div>
  {%- endif %}
{% endmacro %}

{% macro form_errors(form) %}
  {%- if form.errors %}
  <div class="alert alert-danger alert-dismissible text-left" role="alert">
  <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
      {%- for key,err_obj in form.errors.items() recursive%}
          {%  if err_obj is mapping %}
              {{  loop(err_obj.items()) }}
          {% else %}
              {% for single_message in err_obj %}
                <p>{{ single_message }}</p>
              {% endfor %}
          {% endif %}
      {%- endfor %}
  </div>
  {%- endif %}
{% endmacro %}
